==> houses_and_lockersV2.1.tar.gz: houses/house_room.c <==

 /* House_room.c   V2.1      Metaprat       6th April 1992  */

 /* This object defines all the customisation and movement  */
 /* commands for rooms.  It uses information supplied by    */
 /* the master room (main_hall AKA house.c) for the short,  */
 /* long, contents and exits.                               */


#include "house_defines.h"
#include "array_funcs.c"


/* Everything has to be static, because the main house obj inherits from */
/* this, and uses save_/restore_object().  Static variables are not      */
/* included in the save and restore.                                     */

static object main_hall;       /* controlling room */
static int    this_room_index;


/* standard functions */


id(str) {
   if (main_hall) {
      return main_hall->execu_rooms_id(this_room_index, str);
   }
   return (str == "house room");
}


short() {
   if (main_hall) {
      return main_hall->execu_rooms_short(this_room_index);
   }
   return "house room object";
}


long(str) {
   if (main_hall) {
      main_hall->execu_rooms_long(this_room_index, str);
   }
   else {
      write("house room object.  This shouldn't manifest in the game.\n");
   }
   return 1;
}


reset(arg) {
   if (arg) {
      return;
   }
   if (previous_object() && (previous_object()->confirm_house())) {
      main_hall = previous_object()->query_main_hall();
   }
   this_room_index   = -1;  /* eg not indexed in rooms[] */
   set_light(1);
}


confirm_house() {
   return 1;
}



/* The following section is where we build up the room data from the */
/* main hall.  These should be called just after reset, and before   */
/* anyone has the chance to see the room (or init() is called).      */

set_main_hall(arg) {
   main_hall = arg;
}


set_room_index(arg) {
   this_room_index = arg;
}




/* The following section is player commands */


init() {
   int counter;
   string poss_dirs;

   if (environment(this_player()) != this_object()) {
      return;
   }
   poss_dirs = main_hall->get_valid_directions();
   for (counter = 0; counter < sizeof(poss_dirs); counter ++) {
      add_action("go_direction", poss_dirs[counter]);
   }
   add_action("help",     "help");
   add_action("map",      "map");
   add_action("door_cmd", "open");
   add_action("door_cmd", "close");
   add_action("door_cmd", "knock");
   add_action("furnish_cmd", "pour");
   add_action("furnish_cmd", "cook");
   if (IAMWIZ) {
      add_action("toolkit_cmd", "generate_lpc");
      add_action("toolkit_cmd", "room_stats");
      add_action("toolkit_cmd", "master_stats");
      add_action("toolkit_cmd", "agent_stats");
   }
   if (MYNAME == GETOWNER) {
      add_action("invite",  "invite");
      add_action("door_cmd","unlock");
      if (MYNAME != "guest") {
         add_action("door_cmd",    "lock");
         add_action("banish",      "banish");
         add_action("banish",      "expel");
         add_action("toolkit_cmd", "decorate");
         add_action("toolkit_cmd", "redecorate");
         add_action("toolkit_cmd", "build");
         add_action("toolkit_cmd", "destroy");
         add_action("toolkit_cmd", "furnish");
         add_action("toolkit_cmd", "items");
      }
      else {
         add_action("guest_cmd", "decorate");
         add_action("guest_cmd", "banish");
         add_action("guest_cmd", "expel");
         add_action("guest_cmd", "build");
         add_action("guest_cmd", "destroy");
         add_action("guest_cmd", "furnish");
         add_action("guest_cmd", "lock");
      }
   }
   else {
      add_action("illegal_cmd", "decorate");
      add_action("illegal_cmd", "invite");
      add_action("illegal_cmd", "banish");
      add_action("illegal_cmd", "expel");
      add_action("illegal_cmd", "build");
      add_action("illegal_cmd", "destroy");
      add_action("illegal_cmd", "furnish");
      add_action("illegal_cmd", "lock");
      add_action("illegal_cmd", "unlock");
   }
}



help(str) {
   return HOUSEHELP->help(str);
}



go_direction() {
   string destination;

   if ((main_hall->valid_direction(query_verb())) &&
       (main_hall->get_exit_status(this_room_index,query_verb())==DOOR_NONE)) {
      return 0;
   }
   destination = main_hall->file_name_exit(this_room_index, query_verb());
   if (intp(destination)) {
      /* may return 0 (no such exit), or 1 (warning given) */
      return destination;
   }
   this_player()->move_player( query_verb() + "#" + destination);
   return 1;
}


door_cmd(str) {
   /* do something to the door.  This command uses the state transition */
   /* table defined in DOORSTATES.                                      */
   string which_door;
   int    door_status;

   if (!str) {
       write(capitalize(query_verb()) + " which door?\n");
       return 1;
   }
   which_door = main_hall->extract_direction(str);
   if (!main_hall->valid_direction(which_door)) {
      write("Which exit do you mean?  To specify an exit, use:\n");
      write("   'north door'     'exit south'   etc.\n");
      return 1;
   }
   (DOORSTATES)->manipulate_door(main_hall, this_room_index, which_door);
   return 1;
}




invite(str) {
   object invitation;
   int    permanent;
   string who_invited;

   if (!str) {
      write("Usage: invite who [permanent]\n");
      write("Cost: " + INVITECOST + " for nonpermanent, or " +
            INVITEPERMCOST + " for a permanent invitation.\n");
      return 1;
   }
   if (sscanf(str, "%s permanent", who_invited) == 1) {
      permanent = 1;
   }
   else {
      who_invited = str;
   }
   if ((!permanent && !MONEY->has_money(INVITECOST))      ||
       (permanent  && !MONEY->has_money(INVITEPERMCOST))) {
      return 1;
   }
   if (!find_player(who_invited)) {
      write("Sorry, can't locate " + who_invited + ".\n");
      write("Usage: invite who [permanent]\n");
      write("Cost: " + INVITECOST + " for nonpermanent, or " +
            INVITEPERMCOST + " for a permanent invitation.\n");
      return 1;
   }
   invitation = clone_object(INVITATIONS);
   invitation->set_invitee(MYNAME, permanent);
   move_object(invitation,find_player(who_invited));
   if (permanent) {
      tell_object(find_player(who_invited),
         MYCNAME + " gives you a PERMANENT invitation.\n");
      write("You give " + who_invited + " a permanent invitation.\n");
      MONEY->charge_money(INVITEPERMCOST, "invite");
      log_file(LOGFILE, MYCNAME + " gives " + who_invited + " a permanent " +
               "invitation.\n");
   }
   else {
      tell_object(find_player(who_invited),
         MYCNAME + " gives you an invitation.\n");
      write("You give " + who_invited + " an invitation.\n");
      MONEY->charge_money(INVITECOST, "invite");
   }
   say(MYCNAME + " gives " + capitalize(who_invited) +
       " an invitation to visit.\n");
   return 1;
}



banish(str) {
   object invitation;
   object nextob;

   if (!str) {
      write("Who do you want to expel?\n");
      return 1;
   }
   if (!find_player(str)) {
      write("Sorry, can't locate " + str + ".\n");
      return 1;
   }
   if (environment(find_player(str)) == this_object()) {
      write("You throw " + capitalize(str) + " out of your house.\n");
      say(MYCNAME + " throws " + capitalize(str) +
          " violently out of the house.\n");
      move_object(find_player(str), CHURCH);
      command("look",find_player(str));
      tell_room(find_object(CHURCH),
         capitalize(str) + " lands on the floor with a big *THUMP*\n");
   }
   invitation = first_inventory(find_player(str));
   while (invitation) {
      nextob = next_inventory(invitation);
      if (invitation->id(MYNAME + "s invitation")) {
         tell_object(find_player(str),
            MYCNAME + " repeals an invitation.\n");
         destruct(invitation);
      }
      invitation = nextob;
   }
   return 1;
}


map() {
   object map_ob;

   if (!MONEY->has_money(MAPCOST)) {
      return 1;
   }
   map_ob = clone_object(MAPS);
   map_ob->set_owner(GETOWNER);
   move_object(map_ob, this_player());
   write("You are given a well-drawn map.\n");
   MONEY->charge_money(MAPCOST, "map");
   return 1;
}





/* The following commands are defined in toolkits, which are cloned into the */
/* person's directory, if they are not already there.                        */

toolkit_cmd(str) {
   string discard;
   string tool_file;
   object tool_obj;
   object thisob;

   if (GETOWNER != MYNAME) {
      return 0;
   }
   if (query_verb() == "build")        { tool_file = BUILDER;   }
   if (query_verb() == "generate_lpc") { tool_file = CODEGEN;   }
   if (query_verb() == "decorate")     { tool_file = EDITDESC;  }
   if (query_verb() == "redecorate")   { tool_file = EDITDESC;  }
   if (query_verb() == "items")        { tool_file = EDITITEM;  }
   if (query_verb() == "furnish")      { tool_file = EDITFURN;  }
   if (query_verb() == "destroy")      { tool_file = DESTROYER; }
   if (query_verb() == "room_stats")   { tool_file = ROOMSTATS; }
   if (query_verb() == "agent_stats")  { tool_file = ROOMSTATS; }
   if (query_verb() == "master_stats") { tool_file = ROOMSTATS; }
   if (!tool_file) {
      /* shouldn't occur. */
      write("You feel something strange happen.\n");
      return 1;
   }
   thisob = first_inventory(this_player());
   while (thisob) {
      if (sscanf("/" + file_name(thisob), tool_file + "%s", discard) == 1) {
         /* player already has a toolkit object for this command */
         tool_obj = thisob;
         break;
      }
      thisob = next_inventory(thisob);
   }
   if (!tool_obj) {
      /* we don't have a toolkit, so lets make one.  */
      /* Once thats done, run the command inside it. */
      tool_obj = clone_object(tool_file);
      tool_obj->set_main_hall(main_hall);
      tool_obj->set_room_index(this_room_index);
      move_object(tool_obj, this_player());
      return call_other(tool_obj, query_verb(), str);
   }
   /* the object already exists, so ignore the command */
   return 0;
}



furnish_cmd(str) {
   string furnish_opts;

   furnish_opts = main_hall->get_rooms_furnishings(this_room_index);
   if (!furnish_opts || !pointerp(furnish_opts)) {
      return;
   }
   if (search_array(furnish_opts, "kitchen")) {
      if ((query_verb() == "pour") && ALLOW_DRINK) {
         return SUSTENANCE->order(str);
      }
      if ((query_verb() == "cook") && ALLOW_FOOD) {
         return SUSTENANCE->order(str);
      }
   }
}




/* The following are used to block off restricted commands. */

illegal_cmd() {
   write("Sorry, this is not your house, and '" + query_verb() +
         "' is restricted to the owner.\n");
   return 1;
}


guest_cmd() {
   write("Sorry, but you may not use that command if you are the guest\n");
   write("character.  However, you can still 'invite' people, and unlock\n");
   write("doors.  I suggest that you get a character of your own, and\n");
   write("buy a house, so that you can use it to the fullest extent.\n");
   return 1;
}


/* Some querys, used by the toolkit objects to work out which room to */
/* modify.                                                            */

query_room_index() {
   return this_room_index;
}


query_main_hall() {
   return main_hall;
}

==> houses_and_lockersV2.2.tar.gz: houses/house_room.c <==

 /* House_room.c   V2.1      Metaprat       6th April 1992  */

 /* This object defines all the customisation and movement  */
 /* commands for rooms.  It uses information supplied by    */
 /* the master room (main_hall AKA house.c) for the short,  */
 /* long, contents and exits.                               */


#include "house_defines.h"
#include "array_funcs.c"


/* Everything has to be static, because the main house obj inherits from */
/* this, and uses save_/restore_object().  Static variables are not      */
/* included in the save and restore.                                     */

static object main_hall;       /* controlling room */
static int    this_room_index;

static object monster_ob;
static int    natural_light;


/* standard functions */


id(str) {
   if (main_hall) {
      return main_hall->execu_rooms_id(this_room_index, str);
   }
   return (str == "house room");
}


short() {
   if (main_hall) {
      return main_hall->execu_rooms_short(this_room_index);
   }
   return "house room object";
}


long(str) {
   if (main_hall) {
      main_hall->execu_rooms_long(this_room_index, str);
   }
   else {
      write("house room object.  This shouldn't manifest in the game.\n");
   }
   return 1;
}


reset(arg) {
   if (!arg) {
      if (previous_object() && (previous_object()->confirm_house())) {
         main_hall = previous_object()->query_main_hall();
      }
      this_room_index = -1;  /* eg not indexed in rooms[] */
      natural_light = 1;
      set_light(natural_light);
   }
   else {
      if ((!main_hall) || (!main_hall->query_owner())) {
         return;
      }
      if (search_array(main_hall->get_rooms_furnishings(this_room_index),
                       "hourglass")) {
         tell_room(this_object(),
            "You are startled by the noise of the hourglass " +
            "turning by itself!\n");
      }
      make_monster();
   }
}


confirm_house() {
   return 1;
}



/* The following section is where we build up the room data from the */
/* main hall.  These should be called just after reset, and before   */
/* anyone has the chance to see the room (or init() is called).      */

set_main_hall(arg) {
   main_hall = arg;
}


set_room_index(arg) {
   this_room_index = arg;
   make_monster();
}



make_monster() {
   /* makes the rooms monster, also sets up light levels */
   string monster_furnish;
   int    new_light;

   if (!main_hall || (this_room_index < 0)) {
      return;
   }

   monster_furnish = main_hall->get_rooms_furnishings(this_room_index);
   if ((!monster_furnish) || (!pointerp(monster_furnish))) {
      return;
   }

   new_light = !search_array(monster_furnish, "darkness");
   set_light(new_light - natural_light);
   natural_light = new_light;

   /* Now do the monster */
   if (!ALLOW_MONSTER || !this_room_index || monster_ob) {
      return;
   }
   monster_ob = FURNISHOB->make_monster(monster_furnish);
   if (monster_ob) {
      move_object(monster_ob, this_object());
   }
}




/* The following section is player commands */


init() {
   int counter;
   string poss_dirs;

   if (environment(this_player()) != this_object()) {
      return;
   }
   poss_dirs = main_hall->get_valid_directions();
   for (counter = 0; counter < sizeof(poss_dirs); counter ++) {
      add_action("go_direction", poss_dirs[counter]);
   }
   add_action("help",     "help");
   add_action("map",      "map");
   add_action("rank",     "rank");
   add_action("door_cmd", "open");
   add_action("door_cmd", "close");
   add_action("door_cmd", "knock");
   add_action("furnish_cmd", "pour");          /* kitchen     */
   add_action("furnish_cmd", "cook");          /* kitchen     */
   add_action("furnish_cmd", "concentrate");   /* magicmirror */
   if (IAMWIZ) {
      add_action("toolkit_cmd", "generate_lpc");
      add_action("toolkit_cmd", "room_stats");
      add_action("toolkit_cmd", "master_stats");
      add_action("toolkit_cmd", "agent_stats");
      add_action("toolkit_cmd", "show_active_houses");
      add_action("toolkit_cmd", "show_house_owners");
      add_action("toolkit_cmd", "show_public_houses");
      add_action("toolkit_cmd", "show_revenue");
      add_action("toolkit_cmd", "voting_stats");
   }
   if (MYNAME == GETOWNER) {
      add_action("invite",  "invite");
      add_action("door_cmd","unlock");
      if (MYNAME != "guest") {
         add_action("door_cmd",    "lock");
         add_action("banish",      "banish");
         add_action("banish",      "expel");
         add_action("toolkit_cmd", "decorate");
         add_action("toolkit_cmd", "redecorate");
         add_action("toolkit_cmd", "build");
         add_action("toolkit_cmd", "destroy");
         add_action("toolkit_cmd", "furnish");
         add_action("toolkit_cmd", "items");
      }
      else {
         add_action("guest_cmd", "decorate");
         add_action("guest_cmd", "banish");
         add_action("guest_cmd", "expel");
         add_action("guest_cmd", "build");
         add_action("guest_cmd", "destroy");
         add_action("guest_cmd", "furnish");
         add_action("guest_cmd", "lock");
      }
   }
   else {
      add_action("illegal_cmd", "decorate");
      add_action("illegal_cmd", "invite");
      add_action("illegal_cmd", "banish");
      add_action("illegal_cmd", "expel");
      add_action("illegal_cmd", "build");
      add_action("illegal_cmd", "destroy");
      add_action("illegal_cmd", "furnish");
      add_action("illegal_cmd", "lock");
      add_action("illegal_cmd", "unlock");
   }
}



help(str) {
   MASTERHOUSE->inc_wizlist(GETOWNER);
   return HOUSEHELP->help(str);
}


rank() {
   MASTERHOUSE->inc_wizlist(GETOWNER);
   MASTERHOUSE->show_wizlist(MYNAME);
   return 1;
}



go_direction() {
   string destination;

   if ((main_hall->valid_direction(query_verb())) &&
       (main_hall->get_exit_status(this_room_index,query_verb())==DOOR_NONE)) {
      return 0;
   }
   destination = main_hall->file_name_exit(this_room_index, query_verb());
   if (intp(destination)) {
      /* may return 0 (no such exit), or 1 (warning given) */
      return destination;
   }
   MASTERHOUSE->inc_wizlist(GETOWNER);
   this_player()->move_player( query_verb() + "#" + destination);
   return 1;
}


door_cmd(str) {
   /* do something to the door.  This command uses the state transition */
   /* table defined in DOORSTATES.                                      */
   string which_door;
   int    door_status;

   if (!str) {
       write(capitalize(query_verb()) + " which door?\n");
       return 1;
   }
   which_door = main_hall->extract_direction(str);
   if (!main_hall->valid_direction(which_door)) {
      write("Which exit do you mean?  To specify an exit, use:\n");
      write("   'north door'     'exit south'   etc.\n");
      return 1;
   }
   MASTERHOUSE->inc_wizlist(GETOWNER);
   (DOORSTATES)->manipulate_door(main_hall, this_room_index, which_door);
   return 1;
}




invite(str) {
   object invitation;
   int    permanent;
   string who_invited;

   if (!str) {
      write("Usage: invite who [permanent]\n");
      write("Cost: " + INVITECOST + " for nonpermanent, or " +
            INVITEPERMCOST + " for a permanent invitation.\n");
      return 1;
   }
   if (sscanf(str, "%s permanent", who_invited) == 1) {
      permanent = 1;
   }
   else {
      who_invited = str;
   }
   if ((!permanent && !MONEY->has_money(INVITECOST))      ||
       (permanent  && !MONEY->has_money(INVITEPERMCOST))) {
      return 1;
   }
   if (!find_player(who_invited)) {
      write("Sorry, can't locate " + who_invited + ".\n");
      write("Usage: invite who [permanent]\n");
      write("Cost: " + INVITECOST + " for nonpermanent, or " +
            INVITEPERMCOST + " for a permanent invitation.\n");
      return 1;
   }
   invitation = clone_object(INVITATIONS);
   invitation->set_invitee(MYNAME, permanent);
   move_object(invitation,find_player(who_invited));
   if (permanent) {
      tell_object(find_player(who_invited),
         MYCNAME + " gives you a PERMANENT invitation.\n");
      write("You give " + who_invited + " a permanent invitation.\n");
      MONEY->charge_money(INVITEPERMCOST, "invite");
      log_file(LOGFILE, MYCNAME + " gives " + who_invited + " a permanent " +
               "invitation.\n");
   }
   else {
      tell_object(find_player(who_invited),
         MYCNAME + " gives you an invitation.\n");
      write("You give " + who_invited + " an invitation.\n");
      MONEY->charge_money(INVITECOST, "invite");
   }
   say(MYCNAME + " gives " + capitalize(who_invited) +
       " an invitation to visit.\n");
   return 1;
}



banish(str) {
   object invitation;
   object nextob;

   if (!str) {
      write("Who do you want to expel?\n");
      return 1;
   }
   if (!find_player(str)) {
      write("Sorry, can't locate " + str + ".\n");
      return 1;
   }
   if (environment(find_player(str)) == this_object()) {
      write("You throw " + capitalize(str) + " out of your house.\n");
      say(MYCNAME + " throws " + capitalize(str) +
          " violently out of the house.\n");
      move_object(find_player(str), CHURCH);
      command("look",find_player(str));
      tell_room(find_object(CHURCH),
         capitalize(str) + " lands on the floor with a big *THUMP*\n");
   }
   invitation = first_inventory(find_player(str));
   while (invitation) {
      nextob = next_inventory(invitation);
      if (invitation->id(MYNAME + "s invitation")) {
         tell_object(find_player(str),
            MYCNAME + " repeals an invitation.\n");
         destruct(invitation);
      }
      invitation = nextob;
   }
   return 1;
}


map() {
   object map_ob;

   if (!MONEY->has_money(MAPCOST)) {
      return 1;
   }
   map_ob = clone_object(MAPS);
   map_ob->set_owner(GETOWNER);
   move_object(map_ob, this_player());
   write("You are given a well-drawn map.\n");
   MASTERHOUSE->inc_wizlist(GETOWNER);
   MONEY->charge_money(MAPCOST, "map");
   return 1;
}





/* The following commands are defined in toolkits, which are cloned into the */
/* person's directory, if they are not already there.                        */

toolkit_cmd(str) {
   string discard;
   string tool_file;
   object tool_obj;
   object thisob;

   if (GETOWNER != MYNAME) {
      return 0;
   }
   if (query_verb() == "build")              { tool_file = BUILDER;   }
   if (query_verb() == "generate_lpc")       { tool_file = CODEGEN;   }
   if (query_verb() == "decorate")           { tool_file = EDITDESC;  }
   if (query_verb() == "redecorate")         { tool_file = EDITDESC;  }
   if (query_verb() == "items")              { tool_file = EDITITEM;  }
   if (query_verb() == "furnish")            { tool_file = EDITFURN;  }
   if (query_verb() == "destroy")            { tool_file = DESTROYER; }
   if (query_verb() == "room_stats")         { tool_file = ROOMSTATS; }
   if (query_verb() == "agent_stats")        { tool_file = ROOMSTATS; }
   if (query_verb() == "master_stats")       { tool_file = ROOMSTATS; }
   if (query_verb() == "show_active_houses") { tool_file = ROOMSTATS; }
   if (query_verb() == "show_house_owners")  { tool_file = ROOMSTATS; }
   if (query_verb() == "show_public_houses") { tool_file = ROOMSTATS; }
   if (query_verb() == "show_revenue")       { tool_file = ROOMSTATS; }
   if (query_verb() == "voting_stats")       { tool_file = ROOMSTATS; }
   if (!tool_file) {
      /* shouldn't occur. */
      write("You feel something strange happen.\n");
      return 1;
   }
   thisob = first_inventory(this_player());
   while (thisob) {
      if (sscanf("/" + file_name(thisob), tool_file + "%s", discard) == 1) {
         /* Player already has a toolkit object for this command. */
         /* Let the object's action deal with it instead.         */
         return 0;
      }
      thisob = next_inventory(thisob);
   }
   /* we don't have a toolkit, so lets make one.  */
   /* Once thats done, run the command inside it. */
   tool_obj = clone_object(tool_file);
   tool_obj->set_main_hall(main_hall);
   tool_obj->set_room_index(this_room_index);
   move_object(tool_obj, this_player());
   return call_other(tool_obj, query_verb(), str);
}



furnish_cmd(str) {
   string furnish_opts;

   furnish_opts = main_hall->get_rooms_furnishings(this_room_index);
   if (!furnish_opts || !pointerp(furnish_opts)) {
      return;
   }
   if ((query_verb() == "pour")               &&
       ALLOW_DRINK                            &&
       search_array(furnish_opts, "kitchen")) {
      MASTERHOUSE->inc_wizlist(GETOWNER);
      return SUSTENANCE->order(str);
   }
   if ((query_verb() == "cook")               &&
       ALLOW_FOOD                             &&
       search_array(furnish_opts, "kitchen")) {
      MASTERHOUSE->inc_wizlist(GETOWNER);
      return SUSTENANCE->order(str);
   }
   if ((query_verb() == "concentrate")           &&
       ALLOW_MIRROR                              &&
       search_array(furnish_opts, "magicmirror")) {
      FURNISHOB->mirror_cmd(str);
      return 1;
   }
}




/* The following are used to block off restricted commands.  */
/* If notify_fail() is unimplemented on your system, replace */
/* with a 'write' and a return 1;.                           */

illegal_cmd() {
   notify_fail("Sorry, this is not your house, and '" + query_verb() +
               "' is restricted to the owner.\n");
}


guest_cmd() {
   notify_fail(
      "Sorry, but you may not use that command if you are the guest\n" +
      "character.  However, you can still 'invite' people, and unlock\n" +
      "doors.  I suggest that you get a character of your own, and\n" +
      "buy a house, so that you can use it to the fullest extent.\n");
}


/* Some querys, used by the toolkit objects to work out which room to */
/* modify.                                                            */

query_room_index() {
   return this_room_index;
}


query_main_hall() {
   return main_hall;
}
